================================================================================

Supplementary Haskell code for the Journal of Functional Programming article
"Programming Language Semantics: It's Easy As 1,2,3".

Graham Hutton

================================================================================

Arithmetic expressions:

> data Expr = Val Integer | Add Expr Expr
>             deriving Show

Denotational semantics:

> eval :: Expr -> Integer
> eval (Val n)   = n
> eval (Add x y) = eval x + eval y
> 
> eval' :: Expr -> Integer
> eval' = fold id (+)
> 
> fold :: (Integer -> a) -> (a -> a -> a) -> Expr -> a
> fold f g (Val n)   = f n
> fold f g (Add x y) = g (fold f g x) (fold f g y)

Operational semantics:

> trans :: Expr -> [Expr]
> trans (Val n)               = []
> trans (Add (Val n) (Val m)) = [Val (n+m)]
> trans (Add x y)             = [Add x' y | x' <- trans x] ++ [Add x y' | y' <- trans y]
>
> data Tree a = Node a [Tree a]
>               deriving Show
>
> exec :: Expr -> Tree Expr
> exec e = Node e [exec e' | e' <- trans e]
>
> exec' :: Expr -> Tree Expr
> exec' = unfold id trans
> 
> unfold :: (a -> b) -> (a -> [a]) -> a -> Tree b
> unfold f g x = Node (f x) [unfold f g x' | x' <- g x]

Contextual semantics:

> data Con = Hole | AddL Con Expr | AddR Expr Con
>            deriving Show
>
> fill :: Con -> Expr -> Expr
> fill Hole       e = e
> fill (AddL c r) e = Add (fill c e) r
> fill (AddR l c) e = Add l (fill c e)
> 
> split :: Expr -> [(Con,Expr)]
> split e = (Hole,e) : case e of
>    Val n   -> []
>    Add l r -> [(AddL c r, x) | (c,x) <- split l] ++ [(AddR l c, x) | (c,x) <- split r]
>
> reduce :: Expr -> [Expr]
> reduce (Add (Val n) (Val m)) = [Val (n+m)]
> reduce _                     = []
>
> trans' :: Expr -> [Expr]
> trans' e = [fill c x' | (c,x) <- split e, x' <- reduce x]

Big-step semantics:

> eval'' :: Expr -> [Integer]
> eval'' (Val n)   = [n]
> eval'' (Add x y) = [n+m | n <- eval'' x, m <- eval'' y]

================================================================================
